/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ComponentLifeCycle.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.container.lifecycle;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.PetalsException;
import org.ow2.petals.communication.jndi.client.JNDIService;
import org.ow2.petals.container.thread.ComponentLifeCycleThread;
import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.component.context.ComponentInitialContext;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.installation.InstallationContextConstants;
import org.ow2.petals.jbi.management.task.installation.InstallationUtils;
import org.ow2.petals.jbi.messaging.endpoint.ConsumerEndpoint;
import org.ow2.petals.jbi.messaging.exchange.DeliveryChannelImpl;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * Implementation of the <code>ComponentLifeCycle</code> mbean
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBM-WebSourcing
 * @author nsalatge - eBM-WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.container.lifecycle.ComponentLifeCycleMBean.class))
public class ComponentLifeCycle extends LifeCycleAbstract implements ComponentLifeCycleMBean,
        ComponentContextCommunication {

    /**
     * The component linked with the current component life cycle
     */
    protected Component component;

    /**
     * The component description
     */
    private org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription;

    /**
     * The component context
     */
    private ComponentContextImpl componentContext;

    /**
     * The component life cycle thread. Each life cycle have a thread to process
     * operation in the component ClassLoader
     */
    protected ComponentLifeCycleThread componentLifeCycleThread;

    /**
     * Platform component :: SystemState Service
     */
    @Requires(name = "systemstate", signature = org.ow2.petals.platform.systemstate.SystemStateService.class)
    private SystemStateService systemStateService;

    /**
     * Platform Component :: Repository Service
     */
    @Requires(name = "repository", signature = org.ow2.petals.platform.repository.RepositoryService.class)
    protected RepositoryService repositoryService;

    /**
     * The EndpointRegistry is used to manage all the endpoint interfaces in a
     * distributed context
     */
    @Requires(name = "endpoint", signature = org.ow2.petals.jbi.messaging.registry.EndpointRegistry.class)
    protected EndpointRegistry EndpointRegistry;

    /**
     * Communication component :: JNDIService
     */
    @Requires(name = "jndi", signature = org.ow2.petals.communication.jndi.client.JNDIService.class)
    protected JNDIService jndiService;

    protected Map<String, Object> context;

    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * JBI-Management Component :: Admin Service
     */
    @Requires(name = "admin", signature = org.ow2.petals.jbi.management.admin.AdminServiceMBean.class)
    protected AdminServiceMBean adminService;

    /**
     * JBI-Messaging Component :: Router Service
     */
    @Requires(name = "router", signature = org.ow2.petals.jbi.messaging.routing.Router.class)
    protected Router router;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    protected Jbi descriptor;

    protected String installationRootPath;

    protected String workingRootPath;

    protected ConsumerEndpoint address;

    protected InitialContext initialContext;

    /**
     * The component
     */
    @Service
    private org.objectweb.fractal.api.Component comp;

    /**
     * Creates a new instance of {@link ComponentLifeCycle}
     * 
     */
    public ComponentLifeCycle() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.component.lifecycle.ComponentLifeCycleMBean#init(org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription,
     *      javax.jbi.component.Component, javax.management.ObjectName,
     *      org.ow2.petals.jbi.component.thread.ComponentLifeCycleThread,
     *      org.ow2.petals.util.LoggingUtil, java.util.Map)
     */
    public void init(
            org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription,
            Component component, ObjectName mbeanName,
            ComponentLifeCycleThread componentLifeCycleThread, LoggingUtil log,
            Map<String, Object> context) throws PetalsException {

        log.start();

        super.init(mbeanName, log);

        this.componentDescription = componentDescription;

        this.component = component;

        this.context = context;

        this.descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        URI installRoot = (URI) context.get(TaskContextConstants.INSTALL_ROOT);

        this.name = InstallationUtils.getComponentNameFromDescriptor(this.descriptor);

        this.installationRootPath = new File(installRoot).getAbsolutePath();

        this.workingRootPath = repositoryService.getComponentWorkRoot(name).getAbsolutePath();

        this.address = new ConsumerEndpoint(name, containerConfiguration.getName(),
                containerConfiguration.getSubdomain());

        this.initialContext = createComponentInitialContext();

        // 1. create the component context
        this.componentContext = createComponentContext();

        // set the jbiComponent ref to the contextImpl
        if (this.componentContext != null) {
            this.componentContext.setComponent(component);
        }

        // life cycle thread
        this.componentLifeCycleThread = componentLifeCycleThread;

        log.end();
    }

    @Override
    public void doInit() throws JBIException {
        log.start();
        
        componentLifeCycleThread.doInit(componentContext);
        
        log.end();
    }

    @Override
    public void doShutdown() throws JBIException {
        log.start();

        // before shutting down the component, we assure that all SU are shut
        // down
        List<ServiceUnitLifeCycle> suLifeCycles = this.getServiceUnitLifeCycles();
        for (ServiceUnitLifeCycle suLifeCycle : suLifeCycles) {
            if (!SHUTDOWN.equals(suLifeCycle.getCurrentState())) {
                suLifeCycle.shutDown();
            }
        }
        componentLifeCycleThread.doShutdown();
        // close the Channel if required
        DeliveryChannelImpl channel = componentContext.getDeliveryChannelImpl();
        if (channel != null && channel.isOpened())
            channel.close();

        log.end();
    }

    @Override
    public void doStart() throws JBIException {
        log.start();

        componentLifeCycleThread.doStart();

        // in the case of a restart, we restart all the shut down SU LifeCycle
        List<ServiceUnitLifeCycle> suLifeCycles = this.getServiceUnitLifeCycles();
        for (ServiceUnitLifeCycle suLifeCycle : suLifeCycles) {
            if (SHUTDOWN.equals(suLifeCycle.getCurrentState())) {
                suLifeCycle.start();
            }
        }
        log.end();
    }

    @Override
    public void doStop() throws JBIException {
        log.start();
        
        componentLifeCycleThread.doStop();
        
        log.end();
    }

    /**
     * @see javax.jbi.management.ComponentLifeCycleMBean#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() throws JBIException {
        return componentLifeCycleThread.getExtensionMBeanName();
    }

    /**
     * 
     * @return
     */
    public String getName() {
        return this.name;
    }

    @Override
    public void setState(String state) throws JBIException {
        super.setState(state);
        try {
            systemStateService.updateComponentLifeCycleState(this.name, state);
        } catch (Exception e) {
            String msg = "Component state can't be persisted";
            log.error(msg, e);
            throw new JBIException(msg, e);
        }
    }

    /* getters */

    public Component getComponent() {
        return component;
    }

    public ComponentContextImpl getComponentContext() {
        return componentContext;
    }

    public org.ow2.petals.jbi.descriptor.original.generated.Component getComponentDescription() {
        return componentDescription;
    }

    /**
     * Get the consumer endpoint
     * 
     * @return the consumer endpoint
     */
    public ConsumerEndpoint getAddress() {
        return this.address;
    }

    /**
     * Get the endpoint service
     * 
     * @return the endpoint service
     */
    public EndpointRegistry getEndpointRegistry() {
        return this.EndpointRegistry;
    }

    /**
     * Get the initial context
     * 
     * @return the initial context
     */
    public InitialContext getInitialContext() {
        return this.initialContext;
    }

    /**
     * Get the installation root
     * 
     * @return the installation root
     */
    public String getInstallationRoot() {
        return this.installationRootPath;
    }

    /**
     * Get the JBI descriptor
     * 
     * @return the JBI descriptor
     */
    public Jbi getJBIDescriptor() {
        return this.descriptor;
    }

    /**
     * Get the admin service
     * 
     * @return the admin service
     */
    public AdminServiceMBean getAdminService() {
        return this.adminService;
    }

    /**
     * Get the router
     * 
     * @return the router
     */
    public Router getRouter() {
        return this.router;
    }

    /**
     * Get the workspace root
     * 
     * @return the workspace root
     */
    public String getWorkspaceRoot() {
        return this.workingRootPath;
    }

    /**
     * Get the logger
     * 
     * @return the logger
     */
    public Logger getLogger() {
        return this.logger;
    }

    /**
     * Initialization of the petals component
     * 
     * @throws PetalsException
     * @throws JBIException
     * 
     */
    @LifeCycle(on = LifeCycleType.START)
    public void startComponentLifeCycle() {
        log = new LoggingUtil(logger, logger.getName().substring(
                logger.getName().lastIndexOf(".") + 1));
        log.call();
        containerConfiguration = configurationService.getContainerConfiguration();
    }

    /**
     * Stop of the petals component
     * 
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stopComponentLifeCycle() {
        log.start();

        // there is ambiguity between the STOP of the life-cycle and the STOP of
        // the fractal component, so stop the component only if the life-cycle
        // state is STARTED
        if (STARTED.equals(getCurrentState())) {
            try {
                this.doStop();
            } catch (JBIException e) {
                log.error("Failed to stop the component '" + this.name + "'", e);
            }
        }

        try {
            this.componentLifeCycleThread.shutdownThread();
        } catch (JBIException e) {
            log.error("Failed to stop the life-cycle dedicated thread for the component '" + this.name
                    + "'", e);
        }
        
        log.end();
    }

    /**
     * Get the list of the deployed SU on the component.
     * 
     * @return a list of {@code ServiceUnitLifeCycle}
     */
    public List<ServiceUnitLifeCycle> getServiceUnitLifeCycles() {
        List<ServiceUnitLifeCycle> suLifeCycles = new ArrayList<ServiceUnitLifeCycle>();

        for (ServiceAssemblyLifeCycle saLifeCycle : this.adminService
                .getServiceAssemblyLifeCycles().values()) {
            for (ServiceUnitLifeCycle suLifeCycle : saLifeCycle.getServiceUnitsLifeCycles()
                    .values()) {
                if (suLifeCycle.getTargetComponentName().equals(this.name)) {
                    suLifeCycles.add(suLifeCycle);
                }
            }
        }

        return suLifeCycles;
    }

    @Override
    protected String getJBIArtifactType() {
        return "Component";
    }

    /**
     * Create and return the component context
     * 
     * @return the component context
     * @throws PetalsException
     */
    private ComponentContextImpl createComponentContext() throws PetalsException {
        ComponentContextImpl componentContextImpl = new ComponentContextImpl(comp);
        context.put(InstallationContextConstants.COMPONENT_CONTEXT, componentContextImpl);

        return componentContextImpl;
    }

    /**
     * The initial JNDI context for all component. If this initial context is
     * null, it will be created
     * 
     * @return the initial JNDI context for all component. If this initial
     *         context is null, it will be created
     * @throws PetalsException
     *             if {@link InitialContext} can't be created
     */
    private InitialContext createComponentInitialContext() throws PetalsException {
        ComponentInitialContext componentInitialContext = null;
        try {
            componentInitialContext = new ComponentInitialContext(jndiService.getUsersContext());
        } catch (NamingException e) {
            throw new PetalsException("Error creating InitialContext for components", e);
        }
        return componentInitialContext;
    }
}
