/**
 * PETALS - PETALS Services Platform.
 * Copyright (C) 2007 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: LifeCycleSharedLibrary
 *
 */

package org.ow2.petals.container.lifecycle;

import java.net.URL;

import org.ow2.petals.PetalsException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi.SharedLibrary;


/**
 * The <code>LifeCycle</code> interface of the shared library.
 *
 * @author Nicolas SALATGE - EBM WebSourcing
 */
public interface SharedLibraryLifeCycle {

    /**
     * Initialization of sharedLibraryLifeCycle.
     * @param sl
     *           the shared library
     * @param baseUrls
     *           the base urls
     * @param useParentFirst
     *           use parent first
     */
    void init(SharedLibrary sl, URL[] baseUrls, boolean useParentFirst);

    /**
     * Test the installation flag.
     *
     * @return true if the shared library is already installed
     */
    boolean isLoaded();

    /**
     * Install the jbi shared library. Call the Bootstrap onInstall() and cleanup()
     * methods, register the shared library object, and return its jmxName.
     *
     * @throws PetalsException
     *             the component is already installed
     */
    void loadSharedLibrary() throws PetalsException;


    /**
     * Uninstall the jbi shared library. Call the Bootstrap onUninstall() and
     * cleanup() methods, deregister the jbiComponent object.
     *
     */
    void unLoadSharedLibrary();


    /**
     * Start of the shared library.
     *
     */
    void start();


    /**
     * Stop of the shared library.
     *
     */
    void stop();


}
