/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentServiceImpl.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.deployment;

import static org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jbi.management.DeploymentException;
import javax.jbi.management.LifeCycleMBean;

import org.apache.commons.io.IOUtils;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.PetalsException;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.container.lifecycle.ComponentLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.autoload.AutoLoaderService;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentTaskFactory;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.jbi.management.util.XMLResult;
import org.ow2.petals.jbi.management.util.XMLResult.CauseFramework;
import org.ow2.petals.jbi.management.util.XMLResult.MessageType;
import org.ow2.petals.jbi.management.util.XMLResult.TaskResult;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.systemstate.ServiceAssemblyState;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.processor.TaskProcessor;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.ParameterCheckHelper;

/**
 * JBI deployment service implementation.
 * 
 * @author Olivier Fabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.management.deployment.DeploymentServiceMBean.class))
public class DeploymentServiceImpl implements DeploymentServiceMBean {

    /**
     * logger wrapper
     */
    private LoggingUtil log;

    /**
     * Monolog Logger instance.
     */

    @Monolog(name = "logger")
    private Logger logger;

    /**
     * Package Handler
     */
    private PackageHelper packageHandler;

    /**
     * Platform :: SystemState Service
     */
    @Requires(name = "systemstate", signature = org.ow2.petals.platform.systemstate.SystemStateService.class)
    private SystemStateService systemStateService;

    /**
     * Platform :: Repository Service
     */
    @Requires(name = "repository", signature = org.ow2.petals.platform.repository.RepositoryService.class)
    private RepositoryService repositoryService;

    /**
     * JBI management :: Admin service
     */
    @Requires(name = "admin", signature = org.ow2.petals.jbi.management.admin.AdminServiceMBean.class)
    private AdminServiceMBean adminService;

    /**
     * JBI management : AutoLoader Service.
     */
    @Requires(name = "autoloader", signature = org.ow2.petals.jbi.management.autoload.AutoLoaderService.class)
    protected AutoLoaderService autoLoaderService;

    /**
     * JBI messaging :: Endpoint Service
     */
    @Requires(name = "endpoint", signature = org.ow2.petals.jbi.messaging.registry.EndpointRegistry.class)
    private EndpointRegistry EndpointRegistry;

    /**
     * Container :: Container Service
     */
    @Requires(name = "container", signature = org.ow2.petals.container.ContainerService.class)
    protected ContainerService container;

    private DeploymentTaskFactory deploymentTaskFactory;

    /**
     * Default constructor.
     */
    public DeploymentServiceImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#canDeployToComponent(java.lang.String)
     */
    public boolean canDeployToComponent(String componentName) {
        this.log.call();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        boolean result = false;
        ComponentLifeCycle clc = this.adminService.getComponentLifeCycleByName(componentName);
        result = clc != null && LifeCycleMBean.STARTED.equals(clc.getCurrentState())
                && clc.getComponent().getServiceUnitManager() != null;

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getComponentsForDeployedServiceAssembly(java.lang.String)
     */
    public String[] getComponentsForDeployedServiceAssembly(String serviceAssemblyName)
            throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        List<String> result = new ArrayList<String>();
        ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            List<ServiceUnit> sus = saLifeCycle.getServiceAssembly().getServiceUnit();
            for (ServiceUnit unit : sus) {
                result.add(unit.getTarget().getComponentName());
            }
        } else {
            throw new Exception("Cannot retrieve service assembly with the specified name: "
                    + serviceAssemblyName);
        }

        this.log.end();
        return result.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssemblies()
     */
    public String[] getDeployedServiceAssemblies() throws Exception {
        this.log.call();

        Set<String> sas = this.adminService.getServiceAssemblyLifeCycles().keySet();
        return sas.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssembliesForComponent(java.lang.String)
     */
    public String[] getDeployedServiceAssembliesForComponent(String componentName) throws Exception {
        this.log.start();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        Set<String> result = new HashSet<String>();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
                .getServiceAssemblyLifeCycles();
        for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTarget().getComponentName())) {
                    result.add(salc.getServiceAssembly().getIdentification().getName());
                }
            }
        }

        this.log.end();
        return result.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceUnitList(java.lang.String)
     */
    public String[] getDeployedServiceUnitList(String componentName) throws Exception {
        this.log.start();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        Set<String> result = new HashSet<String>();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
                .getServiceAssemblyLifeCycles();
        for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTarget().getComponentName())) {
                    result.add(unit.getIdentification().getName());
                }
            }
        }

        this.log.end();
        return result.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getServiceAssemblyDescriptor(java.lang.String)
     */
    public String getServiceAssemblyDescriptor(String serviceAssemblyName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String result = "Service Assembly doesn't exist :" + serviceAssemblyName;
        try {
            ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                    .getServiceAssemblyByName(serviceAssemblyName);
            if (saLifeCycle != null) {
                URI installationRoot = saLifeCycle.getInstallationRoot();
                URI jbiXmlUri = installationRoot
                        .resolve(JBI_DESCRIPTOR_RESOURCE.replace("\\", "/"));
                // Load JBI descriptor as string
                result = IOUtils.toString(jbiXmlUri.toURL().openStream());
            }

        } catch (Throwable e) {
            // Catch all throwables to prevent exotic exception type (not easily
            // deserializable) throwing
            throw new Exception(e.getMessage() + "\nCause: " + e.getCause());
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.deployment.DeploymentServiceMBean#getServiceUnitForServiceAssembly(java.lang.String)
     */
    public String[] getServiceUnitForServiceAssembly(String saName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(saName, "saName", this.log);

        Set<String> result;
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
                .getServiceAssemblyLifeCycles();

        ServiceAssemblyLifeCycle saLifeCycle = saLifeCycles.get(saName);

        if (saLifeCycle != null) {
            Map<String, ServiceUnitLifeCycle> suLifeCycles = saLifeCycle
                    .getServiceUnitsLifeCycles();
            result = suLifeCycles.keySet();

        } else {
            String msg = "Cannot retrieve service assembly with the specified name : " + saName;
            this.log.error(msg);
            throw new Exception(msg);
        }

        this.log.end();
        return result.toArray(new String[0]);
    }

    public boolean isDeployedServiceUnit(String componentName, String serviceUnitName) {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceUnitName, "serviceUnitName",
                this.log);
        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        boolean result = false;
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
                .getServiceAssemblyLifeCycles();
        for (Iterator<ServiceAssemblyLifeCycle> iter = saLifeCycles.values().iterator(); iter
                .hasNext()
                && !result;) {
            ServiceAssemblyLifeCycle salc = iter.next();
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
            for (ServiceUnit su : sus) {
                if (componentName.equals(su.getTarget().getComponentName())
                        && serviceUnitName.equals(su.getIdentification().getName())) {
                    result = true;
                }
            }
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getState(java.lang.String)
     */
    public String getState(String serviceAssemblyName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String result = null;
        String msg;
        ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            result = saLifeCycle.getCurrentState();
        } else {
            msg = "Cannot retrieve service assembly with the specified name : "
                    + serviceAssemblyName;
            this.log.error(msg);
            throw new Exception(msg);
        }
        if (!(LifeCycleMBean.SHUTDOWN.equals(result) || LifeCycleMBean.STARTED.equals(result) || LifeCycleMBean.STOPPED
                .equals(result))) {
            msg = "Service Assembly state isn't correctly defined";
            this.log.error(msg);
            throw new Exception(msg);
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#deploy(java.lang.String)
     */
    public String deploy(String serviceAssemblyZipUrl) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyZipUrl, "serviceAssemblyZipUrl",
                this.log);

        String result = null;
        try {
            // Transform URL into URI
            URI archiveURI = this.packageHandler.getPackageURI(serviceAssemblyZipUrl, false);

            // Create deployment execution context
            HashMap<String, Object> deploymentContext = this.createDeploymentContext(archiveURI);

            // Create deployment processor and fill it with tasks
            TaskProcessor processor = this.createDeploymentProcessor(deploymentContext, archiveURI);

            // Launch the process
            Throwable processResult = processor.process();
            if (processResult != null) {
                throw new DeploymentException("The deployment of the Service Assembly '"
                        + serviceAssemblyZipUrl + "' failed.", processResult);
            }

            // Create the result string
            result = ((XMLResult) deploymentContext.get(DeploymentContextConstants.XML_RESULT))
                    .getString();

            this.log.info("Service Assembly '"
                    + (String) deploymentContext.get(TaskContextConstants.ENTITY_NAME)
                    + "' succesfully deployed");

        } catch (Throwable e) {
            this.log.error("", e);
            // Catch all throwable to prevent exotic exception type
            // unserializable for external (JMX client) class-loaders
            StringBuilder msg = new StringBuilder(e.getMessage());
            while (e.getCause() != null) {
                e = e.getCause();
                if (e.getMessage() != null) {
                    msg.append("\nCaused by:").append(e.getMessage());
                }
            }
            throw new Exception(msg.toString());
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
     */
    public String start(String serviceAssemblyName) throws Exception {
        return this.start(serviceAssemblyName, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#stop(java.lang.String)
     */
    public String stop(String serviceAssemblyName) throws Exception {
        return this.stop(serviceAssemblyName, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#shutDown(java.lang.String)
     */
    public String shutDown(String serviceAssemblyName) throws Exception {
        return this.shutDown(serviceAssemblyName, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#undeploy(java.lang.String)
     */
    public String undeploy(String serviceAssemblyName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName, "serviceAssemblyZipUrl",
                this.log);

        String output = null;

        try {
            // Create undeployment execution context
            HashMap<String, Object> undeploymentContext = this
                    .createUndeploymentContext(serviceAssemblyName);

            // Create undeployment processor and fill with tasks
            TaskProcessor processor = this.createUndeploymentProcessor(undeploymentContext);

            // Launch the process
            Throwable processResult = processor.process();
            if (processResult != null) {
                throw new DeploymentException("The undeployment of the Service Assembly '"
                        + serviceAssemblyName + "' failed.", processResult);
            }

            // Create the result string
            output = ((XMLResult) undeploymentContext.get(DeploymentContextConstants.XML_RESULT))
                    .getString();

        } catch (Throwable e) {
            this.log.error("", e);
            // Catch all throwable to prevent exotic exception type
            // unserializable for external (JMX client) class-loaders
            StringBuilder msg = new StringBuilder(e.getMessage());
            while (e.getCause() != null) {
                e = e.getCause();
                if (e.getMessage() != null) {
                    msg.append("\nCaused by:").append(e.getMessage());
                }
            }
            throw new Exception(msg.toString());
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' succesfully undeployed");

        this.log.end();
        return output;
    }

    public String[] startAllServiceAssemblies() {
        this.log.start();

        List<String> startedSAs = new ArrayList<String>();
        /*
         * Try to start all service assemblies, if a SA can't be started, it's
         * skipped and the starting process ins't aborted.
         */
        for (String name : this.systemStateService.getAllDeployedServiceAssemblyNames()) {
            try {
                this.start(name);
                startedSAs.add(name);
            } catch (Throwable e) {
                this.log.warning("The SA '" + name + "' can't be started", e);
            }
        }

        this.log.end();
        return startedSAs.toArray(new String[startedSAs.size()]);
    }

    public String[] stopAllServiceAssemblies() {
        this.log.start();

        List<String> stopedSAs = new ArrayList<String>();
        /*
         * Try to stop all service assemblies. If a SA can't be stopped, it's
         * skipped and the stopping process ins't aborted.
         */
        for (String name : this.systemStateService.getAllDeployedServiceAssemblyNames()) {
            try {
                this.stop(name);
                stopedSAs.add(name);
            } catch (Throwable e) {
                this.log.warning("The SA '" + name + "' can't be stopped", e);
            }
        }

        this.log.end();
        return stopedSAs.toArray(new String[stopedSAs.size()]);
    }

    public String[] shutdownAllServiceAssemblies() {
        this.log.start();

        List<String> shutdownSAs = new ArrayList<String>();
        /*
         * Try to shutdown all service assemblies. If a SA can't be shutdown,
         * it's skipped and the shutting down process ins't aborted.
         */
        for (String name : this.systemStateService.getAllDeployedServiceAssemblyNames()) {
            try {
                this.shutDown(name);
                shutdownSAs.add(name);
            } catch (Throwable e) {
                this.log.warning("The SA '" + name + "' can't be shut down", e);
            }
        }

        this.log.end();
        return shutdownSAs.toArray(new String[shutdownSAs.size()]);
    }

    public String[] undeployAllServiceAssemblies(boolean force) {
        this.log.start();

        List<String> undeployedSAs = new ArrayList<String>();
        /*
         * Try to undeploy all service assemblies. If a SA can't be undeployed,
         * it's skipped and the undeploying process ins't aborted.
         */
        for (String name : this.systemStateService.getAllDeployedServiceAssemblyNames()) {
            if (force) {
                if (this.forceUndeploy(name)) {
                    undeployedSAs.add(name);
                }
            } else {
                try {
                    this.undeploy(name);
                    undeployedSAs.add(name);
                } catch (Throwable e) {
                    this.log.warning("The Service Assembly '" + name + "' can't be undeployed", e);
                }
            }
        }

        this.log.end();
        return undeployedSAs.toArray(new String[undeployedSAs.size()]);
    }

    public boolean forceUndeploy(String serviceAssemblyName) {
        this.log.start();

        boolean removed = false;

        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
                .getServiceAssemblyLifeCycles();

        ServiceAssemblyLifeCycle saLifeCycle = saLifeCycles.get(serviceAssemblyName);

        if (saLifeCycle != null) {
            saLifeCycle.setForcedState(true);
        } else {
            this.log.warning("Cannot retrieve Service Assembly with the specified name : "
                    + serviceAssemblyName);
        }

        try {
            if (!LifeCycleMBean.SHUTDOWN.equals(saLifeCycle.getCurrentState())) {
                this.shutDown(serviceAssemblyName, true);
            }
        } catch (Exception e) {
            this.log.warning("An error occured during Service Assembly shut down", e);
        }
        try {
            this.undeploy(serviceAssemblyName);
            removed = true;
        } catch (Exception e) {
            // do nothing
        }

        this.log.end();
        return removed;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.PetalsLifeCycle#shutdown()
     */
    public void shutdown() throws Exception {
        this.log.call();
        // Undeploy all service assemblies
        this.undeployAllServiceAssemblies(true);
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.start();

        this.packageHandler = new PackageHelper(this.logger, this.repositoryService);

        this.deploymentTaskFactory = new DeploymentTaskFactory(this.packageHandler,
                this.EndpointRegistry, this, this.systemStateService, this.repositoryService,
                this.log, this.adminService, this.container, this.autoLoaderService);

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

    /**
     * Start the given service assembly. if {@code forceState=true}, the SA
     * would be in the {@code STARTED} state even errors occur in starting of
     * service units, otherwise the SA get back to its previous state.
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
     * 
     * @param serviceAssemblyName
     * @param forcedState
     * @return
     * @throws Exception
     */
    private String start(String serviceAssemblyName, boolean forcedState) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String output = null;
        try {
            XMLResult xmlResult = new XMLResult("start");
            ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                    .getServiceAssemblyByName(serviceAssemblyName);
            if (saLifeCycle != null) {
                saLifeCycle.setForcedState(forcedState);
                saLifeCycle.start();
            } else {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not deployed.");
            }
            xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                    "Successfully started service assembly : {1}",
                    new String[] { serviceAssemblyName }, CauseFramework.YES);
            output = xmlResult.getString();
        } catch (Throwable e) {
            this.log.error("", e);
            // Catch all throwable to prevent exotic exception type
            // unserializable for external (JMX client) class-loaders
            StringBuilder msg = new StringBuilder(e.getMessage());
            while (e.getCause() != null) {
                e = e.getCause();
                if (e.getMessage() != null) {
                    msg.append("\nCaused by: ").append(e.getMessage());
                }
            }
            throw new Exception(msg.toString());
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' succesfully started");

        this.log.end();
        return output;

    }

    private String stop(String serviceAssemblyName, boolean forcedState) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String output = null;
        try {
            ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                    .getServiceAssemblyByName(serviceAssemblyName);
            if (saLifeCycle != null) {
                saLifeCycle.setForcedState(forcedState);
                saLifeCycle.stop();
            } else {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not deployed.");
            }
            XMLResult xmlResult = new XMLResult("stop");
            xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                    "Successfully stopped service assembly : {1}",
                    new String[] { serviceAssemblyName }, CauseFramework.YES);
            output = xmlResult.getString();

        } catch (Throwable e) {
            this.log.error("", e);
            // Catch all throwable to prevent exotic exception type
            // unserializable for external (JMX client) class-loaders
            StringBuilder msg = new StringBuilder(e.getMessage());
            while (e.getCause() != null) {
                e = e.getCause();
                if (e.getMessage() != null) {
                    msg.append("\nCaused by: ").append(e.getMessage());
                }
            }
            throw new Exception(msg.toString());
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' succesfully stopped");

        this.log.end();
        return output;
    }

    /**
     * Shut down the given service assembly. if {@code forceState=true}, the SA
     * would be in the {@code SHUTDOWN} state even errors occur in starting of
     * service units, otherwise the SA get back to its previous state.
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
     * 
     * @param serviceAssemblyName
     * @param forcedState
     * @return
     * @throws Exception
     */
    private String shutDown(String serviceAssemblyName, boolean forcedState) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String output = null;
        try {
            XMLResult xmlResult = new XMLResult("shutdown");

            ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                    .getServiceAssemblyByName(serviceAssemblyName);
            if (saLifeCycle != null) {
                saLifeCycle.setForcedState(forcedState);
                saLifeCycle.shutDown();
            } else {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not deployed.");
            }
            xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                    "Successfully shutdown service assembly : {1}",
                    new String[] { serviceAssemblyName }, CauseFramework.YES);
            output = xmlResult.getString();
        } catch (Throwable e) {
            this.log.error("", e);
            // Catch all throwable to prevent exotic exception type
            // unserializable for external (JMX client) class-loaders
            StringBuilder msg = new StringBuilder(e.getMessage());
            while (e.getCause() != null) {
                e = e.getCause();
                if (e.getMessage() != null) {
                    msg.append("\nCaused by: ").append(e.getMessage());
                }
            }
            throw new Exception(msg.toString());
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' succesfully shut down");

        this.log.end();
        return output;
    }

    /**
     * Create the deployment context
     * 
     * @param archiveURI
     *            URI of the Service Assembly installation package file; must be
     *            non-null, non-empty, and a legal URI.
     * @return the Deployment Context HashMap
     */
    private HashMap<String, Object> createDeploymentContext(URI archiveURI) {
        this.log.call();

        HashMap<String, Object> deploymentContext = new HashMap<String, Object>();
        deploymentContext.put(TaskContextConstants.ARCHIVE_URI, archiveURI);

        return deploymentContext;
    }

    /**
     * Create the deployment processor and add all required task for the
     * complete deployment of a service assembly
     * 
     * @param deploymentContext
     *            the deployment context
     * @param archiveURI
     *            the sl archive URI
     * @return the filled deployment processor
     */
    private TaskProcessor createDeploymentProcessor(HashMap<String, Object> deploymentContext,
            URI archiveURI) {
        this.log.call();

        TaskProcessor processor = new TaskProcessor(deploymentContext, this.log);

        String scheme = archiveURI.getScheme();
        if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("ftp")) {
            processor.addTask(this.deploymentTaskFactory.getDownloadTask());
            processor.addTask(this.deploymentTaskFactory.getExtractTask());
        } else if (scheme.equalsIgnoreCase("file")) {
            if (this.packageHandler.isDirectory(archiveURI)) {
                processor.addTask(this.deploymentTaskFactory.getCopyArchiveToWorkDirTask());
            } else {
                processor.addTask(this.deploymentTaskFactory.getExtractTask());
            }
        } else {
            throw new IllegalArgumentException("URL protocol not supported:" + scheme);
        }
        processor.addTask(this.deploymentTaskFactory.getSaCheckPackageTask());
        processor.addTask(this.deploymentTaskFactory.getCopySAPackageToRepositoryTask());
        processor.addTask(this.deploymentTaskFactory.getCreateXMLDeploymentResultTask());
        if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("ftp")) {
            processor.addTask(this.deploymentTaskFactory.getExtractSUsTask());
        } else if (scheme.equalsIgnoreCase("file")) {
            if (!this.packageHandler.isDirectory(archiveURI)) {
                processor.addTask(this.deploymentTaskFactory.getExtractSUsTask());
            } else {
                processor.addTask(this.deploymentTaskFactory.getRenameSUsTask());
            }
        }
        processor.addTask(this.deploymentTaskFactory.getRegisterSALifeCycleTask());
        processor.addTask(this.deploymentTaskFactory.getRegisterAllConnectionTask());
        processor.addTask(this.deploymentTaskFactory.getDeployAllSUTask());
        processor.addTask(this.deploymentTaskFactory.getCreateSAStateHolderTask());
        if (scheme.equalsIgnoreCase("file") && !this.packageHandler.isDirectory(archiveURI)) {
            processor.addTask(this.deploymentTaskFactory.getCopyArchiveToInstalledDirTask());
        }
        processor.addTask(this.deploymentTaskFactory.getSuccessTask());

        return processor;
    }

    /**
     * Create the undeployment processor and add all required task for the
     * complete undeployment of a service assembly
     * 
     * @param undeploymentContext
     *            the deployment context
     * @return filled undeployment processor
     */
    private TaskProcessor createUndeploymentProcessor(HashMap<String, Object> undeploymentContext) {
        this.log.call();

        TaskProcessor processor = new TaskProcessor(undeploymentContext, this.log);

        processor.addTask(this.deploymentTaskFactory.getCreateXMLUndeploymentResultTask());
        processor.addTask(this.deploymentTaskFactory.getRetrieveSALifeCycleTask());
        processor.addTask(this.deploymentTaskFactory.getUndeployAllSUTask());
        processor.addTask(this.deploymentTaskFactory.getRemoveAllConnectionsTask());
        processor.addTask(this.deploymentTaskFactory.getUnregisterSALifeCycleTask());
        processor.addTask(this.deploymentTaskFactory.getRemoveSAStateHolderTask());
        processor.addTask(this.deploymentTaskFactory.getRemoveSAFolderTask());
        processor.addTask(this.deploymentTaskFactory.getDeleteArchiveFromInstalledDirTask());

        return processor;
    }

    /**
     * Create the undeployment context
     * 
     * @param serviceAssemblyName
     *            the service assembly name
     * @return the undeployment context
     * @throws PetalsException
     *             if error occured during JBI descriptor loading
     */
    private HashMap<String, Object> createUndeploymentContext(String serviceAssemblyName)
            throws PetalsException {
        this.log.call();

        HashMap<String, Object> undeploymentContext = new HashMap<String, Object>();

        ServiceAssemblyState state = this.systemStateService
                .getServiceAssemblyState(serviceAssemblyName);

        undeploymentContext.put(TaskContextConstants.ENTITY_NAME, serviceAssemblyName);
        undeploymentContext.put(TaskContextConstants.ARCHIVE_URI, this.packageHandler
                .getPackageURI(state.getArchiveURL(), false));
        return undeploymentContext;
    }

}
