/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.net.URI;
import java.util.HashMap;

import org.ow2.petals.PetalsException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.processor.Task;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task copy unzipped entity (component, shared lib, service assembly)
 * package into the repository <br/> Inputs : {@link Jbi}, unzipped
 * package root {@link File} <br/> Outputs : entity install root {@link URI}
 * <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public abstract class AbstractCopyPackageToRepositoryTask implements Task {

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public AbstractCopyPackageToRepositoryTask(RepositoryService repositorySrv, LoggingUtil log) {
        super();
        this.repositorySrv = repositorySrv;
        this.log = log;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        File packageFile = (File) context.get(TaskContextConstants.UNZIP_ROOT);

        /*
         * Explode Comp
         */
        URI installRoot = copyPackage(descriptor, packageFile);

        /*
         * Fill context
         */
        context.put(TaskContextConstants.INSTALL_ROOT, installRoot);

    }

    /**
     * Expand the given package into the system repository
     * 
     * @param descriptor
     *            jbi descriptor of the entity
     * @param packageFile
     *            the entity package root file
     * @return return installation URI of the expanded package, null if it fails
     * @throws ManagementException
     *             if the package copy fails or if returned install root is null
     */
    protected URI copyPackage(Jbi descriptor, File packageFile)
            throws ManagementException {
        String msg;
        URI installationRoot = null;

        try {
            installationRoot = copyEntityPackage(descriptor, packageFile);
        } catch (PetalsException pe) {
            msg = "Unexpected error adding JBI installation " + "package into repository: "
                    + packageFile.getAbsolutePath();
            log.error(msg, pe);
            throw new ManagementException(msg, pe);
        }
        return installationRoot;
    }

    protected abstract URI copyEntityPackage(Jbi descriptor, File packageFile)
            throws PetalsException;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) {
        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        try {
            removeEntityPackage(descriptor);
        } catch (PetalsException e) {
            String msg = "Failed to revert a ExtractTask";
            log.error(msg, e);
        }

    }

    protected abstract void removeEntityPackage(Jbi descriptor) throws PetalsException;

}
