/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RetrieveSAAndSUInstallRootsTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.petals.PetalsException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.processor.Task;

/**
 * This task retrieves the SA install root and all SUs install roots <br/>
 * Inputs : {@link Jbi} <br/> Outputs : SA install root {@link URI},
 * SU install roots {@link Map} <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class RetrieveSAAndSUInstallRootsTask implements Task {

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositoryService;

    /**
     * Package Handler
     */
    protected PackageHelper packageHandler;

    public RetrieveSAAndSUInstallRootsTask(RepositoryService repositoryService,
        PackageHelper packageHandler) {
        super();
        this.repositoryService = repositoryService;
        this.packageHandler = packageHandler;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        Jbi descriptor = (Jbi) context
            .get(TaskContextConstants.DESCRIPTOR);

        URI installationRoot = repositoryService.getServiceAssemblyInstallRoot(
            DeploymentUtils.getServiceAssemblyName(descriptor)).toURI();
        Map<String, URI> suInstallRoots = retrieveSUInstallRoots(
            installationRoot, descriptor);

        // Fill context
        context.put(TaskContextConstants.INSTALL_ROOT, installationRoot);
        context
            .put(DeploymentContextConstants.SU_INSTALL_ROOTS, suInstallRoots);

    }

    /**
     * Retrieve all SU installation root {@link URI}
     * 
     * @param descriptor
     *            the service assembly descriptor
     * @param installationRoot
     *            the SA installation root {@link URI}
     * @return the service unit install roots {@link Map}
     * @throws PetalsException 
     */
    protected Map<String, URI> retrieveSUInstallRoots(URI installationRoot,
        Jbi descriptor) throws PetalsException {
        Map<String, URI> suInstallRoots = new HashMap<String, URI>();
        List<ServiceUnit> sus = descriptor.getServiceAssembly()
            .getServiceUnit();
        for (ServiceUnit unit : sus) {
            String suName = unit.getIdentification().getName();
            suInstallRoots.put(suName, packageHandler.getPackageURI(
                "file://" + installationRoot.getPath() + suName, false));
        }
        return suInstallRoots;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) {
        // Nothing to do

    }

}
