/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.install.component;

import java.util.HashMap;

import javax.jbi.JBIException;
import javax.management.ObjectName;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.util.Fractal;
import org.ow2.petals.PetalsException;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.container.lifecycle.Installer;
import org.ow2.petals.container.lifecycle.InstallerMBean;
import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.MBeanNames;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.installation.InstallationContextConstants;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.kernel.server.FractalHelper;
import org.ow2.petals.processor.Task;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task creates the InstallerMBean for a component and register it in the
 * JMX server <br/> Inputs : {@link Jbi}, component install root URI,
 * {@link ComponentContextImpl} <br/> Outputs : component installer
 * {@link ObjectName} <br/>
 * 
 * @author ofabre
 * 
 */
public class CreateAndRegisterInstallerMBeanTask implements Task {
    /**
     * logger wrapper
     */
    private LoggingUtil log;

    /**
     * JBI Container :: JMX Agent Admin
     */
    private AdminServiceMBean adminService;

    private Router router;

    protected ContainerService containerService;

    /**
     * Creates a new instance of {@link CreateAndRegisterInstallerMBeanTask}
     * 
     * @param log
     * @param jmxAdmin
     * @param logger
     * @param router
     * @param containerService
     */
    public CreateAndRegisterInstallerMBeanTask(LoggingUtil log, AdminServiceMBean jmxAdmin,
            Router router, ContainerService container) {
        super();
        this.log = log;
        this.adminService = jmxAdmin;
        this.router = router;
        this.containerService = container;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        ObjectName result = createAndRegisterInstallerMBean(router, context);
        context.put(InstallationContextConstants.INSTALLER_OBJECT_NAME, result);
    }

    /**
     * Create and register a new InstallerMBean for the loaded component
     * 
     * @param componentContext
     *            the new Component execution context
     * @param descriptor
     *            the component Jbi
     * @param installationRoot
     *            the URI representing the component installation folder
     * @return the new InstallerMBean name
     * @throws PetalsException
     */
    private ObjectName createAndRegisterInstallerMBean(Router router, HashMap<String, Object> ctxt)
            throws PetalsException {
        ObjectName result = null;

        Jbi descriptor = (Jbi) ctxt.get(TaskContextConstants.DESCRIPTOR);

        org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription = descriptor.getComponent();
        try {
            MBeanNames mbeanNames = adminService.getMBeanNames();
            result = mbeanNames.createInstallerMBeanName(componentDescription.getIdentification()
                    .getName());

            Component installer = containerService.createInstaller(componentDescription);

            // Start the new installer
            FractalHelper.getFractalHelper().startComponent(installer);

            ((InstallerMBean) installer.getFcInterface("service")).init(ctxt);

            adminService.registerInstaller((Installer) installer.getFcInterface("/content"));
        } catch (ManagementException e) {
            throw new PetalsException("Error creating or registring installerMBean", e);
        } catch (ADLException e) {
            throw new PetalsException("Error creating or registring installerMBean", e);
        } catch (NoSuchInterfaceException e) {
            throw new PetalsException("Error creating or registring installerMBean", e);
        } catch (IllegalLifeCycleException e) {
            throw new PetalsException("Error creating or registring installerMBean", e);
        } catch (JBIException e) {
            throw new PetalsException("Error creating or registring installerMBean", e);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        ObjectName installerMBeanName = (ObjectName) context
                .get(InstallationContextConstants.INSTALLER_OBJECT_NAME);

        // Reverse installer MBean registration
        if (installerMBeanName != null) {
            try {
                adminService.unregisterInstaller(installerMBeanName);

                // remove the fractal component
                // search the component controller
                org.objectweb.fractal.api.Component container = containerService
                        .getParentContainer();
                ContentController containerContentController = Fractal
                        .getContentController(container);

                // Get the component lifecycle component
                Jbi descriptor = (Jbi) context
                        .get(TaskContextConstants.DESCRIPTOR);
                org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription = descriptor.getComponent();
                org.objectweb.fractal.api.Component componentInstaller = FractalHelper
                        .getFractalHelper().getComponentByName(containerContentController,
                                "Installer_" + componentDescription.getIdentification().getName());

                // Stop the component lifecycle component
                FractalHelper.getFractalHelper().stopComponent(componentInstaller);

                // Remove the component lifecycle component
                containerContentController.removeFcSubComponent(componentInstaller);

            } catch (ADLException e) {
                // Nothing to do just log
                log.warning("Component installer can't be unregistered or doesn't exist");
            } catch (NoSuchInterfaceException e) {
                // Nothing to do just log
                log.warning("Component installer can't be unregistered or doesn't exist");
            } catch (IllegalLifeCycleException e) {
                // Nothing to do just log
                log.warning("Component installer can't be unregistered or doesn't exist");
            } catch (IllegalContentException e) {
                // Nothing to do just log
                log.warning("Component installer can't be unregistered or doesn't exist");
            } catch (ManagementException e) {
                // Nothing to do just log
                log.warning("Component installer can't be unregistered or doesn't exist");
            }
        }
    }
}
