/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.install.sl;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.ow2.petals.PetalsException;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.ClassLoaderDelegationType;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi.SharedLibrary;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.installation.InstallationUtils;
import org.ow2.petals.kernel.server.FractalHelper;
import org.ow2.petals.platform.classloader.ClassLoaderService;
import org.ow2.petals.processor.Task;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task create the shared lib classloader <br/> Inputs :
 * {@link Jbi}, component install root URI <br/>
 * 
 * @author ofabre
 * 
 */
public class CreateSLClassloaderTask implements Task {

    /**
     * logger wrapper
     */
    private LoggingUtil log;

    /**
     * Platform Component :: Loader Service
     */
    private ClassLoaderService loaderSrv;

    /**
     * JBI Admin Service :: JMX Agent Admin
     */
    private AdminServiceMBean adminService;

    /**
     * The Petals Container
     */
    protected ContainerService containerService;

    public CreateSLClassloaderTask(LoggingUtil log, ClassLoaderService loaderSrv,
            AdminServiceMBean adminService, ContainerService containerService) {
        super();
        this.log = log;
        this.loaderSrv = loaderSrv;
        this.containerService = containerService;
        this.adminService = adminService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        log.start();

        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);
        URI installRoot = (URI) context.get(TaskContextConstants.INSTALL_ROOT);

        // Get shared library production element
        SharedLibrary sl = descriptor.getSharedLibrary();

        // Get classloader delegation mode
        boolean parentFirst = ClassLoaderDelegationType.PARENT_FIRST.equals(sl
                .getClassLoaderDelegation());

        // Create baseUrls Array for shared lib classloader
        List<URL> urls = new ArrayList<URL>();
        try {
            urls.add(installRoot.toURL());
        } catch (MalformedURLException e) {
            throw new PetalsException("Unexpected error creating URL for shared lib classloader", e);
        }
        URL[] baseUrls = urls.toArray(new URL[] {});

        // Create shared lib classloader
        try {
            Component sharedLibraryLifeCycle = containerService.createSharedLibraryLifeCycle(sl);

            // Start the new installer
            FractalHelper.getFractalHelper().startComponent(sharedLibraryLifeCycle);

            ((SharedLibraryLifeCycle) sharedLibraryLifeCycle.getFcInterface("service")).init(sl,
                    baseUrls, parentFirst);

            // load in PetalsShared Libraries
            ((SharedLibraryLifeCycle) sharedLibraryLifeCycle.getFcInterface("service"))
                    .loadSharedLibrary();

            this.adminService.registerSharedLibrary(sl.getIdentification().getName(),
                    ((SharedLibraryLifeCycle) sharedLibraryLifeCycle.getFcInterface("service")));

            // loaderSrv.createSharedLibraryClassLoader(slName, baseUrls, sl
            // .getSharedLibraryClassPath(), parentFirst);
        } catch (PetalsException pe) {
            throw new PetalsException("Unexpected error creating class loader", pe);
        }

        log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        log.call();

        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);
        loaderSrv.deleteClassLoader(InstallationUtils.getSLNameFromDescriptor(descriptor));
    }

}
