/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing.module;

import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.topology.TopologyService;
import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.jbi.messaging.routing.RouterServiceImpl;
import org.ow2.petals.jbi.messaging.routing.RoutingException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.transport.Transporter;
import org.ow2.petals.util.LoggingUtil;

/**
 * A router module definition.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 2.2
 * 
 */
public abstract class RouterModule implements Router {

    protected RouterServiceImpl routerService;

    protected Transporter transporter;

    protected ConfigurationService configurationService;

    protected TopologyService topologyService;

    protected EndpointRegistry endpointService;

    protected Logger logger;

    protected LoggingUtil log;

    private boolean init = false;

    /**
     * Creates a new instance of RouterModule
     */
    public RouterModule() {
    }

    /**
     * 
     * @param routerService
     */
    public final void init(RouterServiceImpl routerService) {
        this.routerService = routerService;
        this.transporter = routerService.getTransporter();
        this.configurationService = routerService.getConfigurationService();
        this.topologyService = routerService.getTopologyService();
        this.endpointService = routerService.getEndpointService();
        this.logger = routerService.getLogger();
        this.log = routerService.getLog();
    }

    /**
     * 
     * @throws RouterModuleException
     */
    public void init() throws RouterModuleException {
        if (!init) {
            try {
                doInit();
            } catch (Exception e) {
                throw new RouterModuleException(e);
            } finally {
                init = true;
            }
        } else {
            throw new RouterModuleException("The module is already initialized");
        }
    }

    /**
     * 
     * @throws Exception
     */
    protected abstract void doInit() throws Exception;

    /**
     * 
     * @return
     */
    public String getName() {
        return this.getClass().getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#addComponent(org.ow2.petals.jbi.component.context.ComponentContextImpl)
     */
    public void addComponent(final ComponentContextImpl componentContext) throws RoutingException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#receive(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      long)
     */
    public MessageExchangeImpl receive(final ComponentContextImpl source, final long timeoutMS)
            throws RoutingException {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#removeComponent(org.ow2.petals.jbi.component.context.ComponentContextImpl)
     */
    public void removeComponent(final ComponentContextImpl componentContext)
            throws RoutingException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#send(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, long)
     */
    public void send(final ComponentContextImpl source, final MessageExchangeImpl exchange,
            final long timeOut) throws RoutingException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#sendSync(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, long)
     */
    public MessageExchangeImpl sendSync(final ComponentContextImpl source,
            final MessageExchangeImpl exchange, final long timeOut) throws RoutingException {
        return null;
    }
}
